"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const EffectsUtil_1 = require("../utils/EffectsUtil");
class ColorEffect {
    constructor(config) {
        this._config = config;
    }
    get color() {
        return this._color;
    }
    setColor(color, pattern, message) {
        this._color = color;
        this._colorFunction = this.getColorFunction(pattern, message);
    }
    calculate(input) {
        return `rgba(${this._colorFunction(input)}, 1)`;
    }
    getColorFunction(pattern, message) {
        switch (this.color) {
            case "cyan":
                return () => EffectsUtil_1.CYAN;
            case "flash1":
                return this._config.version === "osrs"
                    ? (input) => this.getFlash(input, EffectsUtil_1.FLASH_1_OSRS)
                    : (input) => this.getFlash(input, EffectsUtil_1.FLASH_1_RS3);
            case "flash2":
                return this._config.version === "osrs"
                    ? (input) => this.getFlash(input, EffectsUtil_1.FLASH_2_OSRS)
                    : (input) => this.getFlash(input, EffectsUtil_1.FLASH_2_RS3);
            case "flash3":
                return this._config.version === "osrs"
                    ? (input) => this.getFlash(input, EffectsUtil_1.FLASH_3_OSRS)
                    : (input) => this.getFlash(input, EffectsUtil_1.FLASH_3_RS3);
            case "glow1":
                return this._config.version === "osrs"
                    ? (input) => this.getGlow(input, EffectsUtil_1.GLOW_1_OSRS)
                    : (input) => this.getGlow(input, EffectsUtil_1.GLOW_1_RS3);
            case "glow2":
                return this._config.version === "osrs"
                    ? (input) => this.getGlow(input, EffectsUtil_1.GLOW_2_OSRS)
                    : (input) => this.getGlow(input, EffectsUtil_1.GLOW_2_RS3);
            case "glow3":
                return this._config.version === "osrs"
                    ? (input) => this.getGlow(input, EffectsUtil_1.GLOW_3_OSRS)
                    : (input) => this.getGlow(input, EffectsUtil_1.GLOW_3_RS3);
            case "green":
                return () => EffectsUtil_1.GREEN;
            case "pattern": {
                const patternColors = this.createPatternColors(pattern, message);
                return (input) => this.getPattern(input, patternColors);
            }
            case "purple":
                return () => EffectsUtil_1.PURPLE;
            case "rainbow": {
                const rainbowColors = this.createRainbowColors(message);
                return (input) => this.getPattern(input, rainbowColors);
            }
            case "red":
                return () => EffectsUtil_1.RED;
            case "white":
                return () => EffectsUtil_1.WHITE;
            case "yellow":
                return () => EffectsUtil_1.YELLOW;
        }
    }
    getFlash({ frame }, flash) {
        return flash[Number(((2 * frame) % this._config.fps) * 2 >= this._config.fps)];
    }
    getGlow({ frame }, glow) {
        const inbetweenFrames = this._config.totalFrames / (glow.length - 1);
        const index = Math.floor(frame / inbetweenFrames);
        const currentColor = glow[index];
        const nextColor = glow[index + 1];
        const incrementFactor = (frame % inbetweenFrames) / inbetweenFrames;
        return currentColor.map((color, index) => color + Math.round((nextColor[index] - color) * incrementFactor));
    }
    getPattern({ index }, pattern) {
        return pattern[index];
    }
    createPatternColors(pattern, message) {
        const buckets = this.createPatternBuckets(pattern.length, message.length);
        return pattern.flatMap((character, index) => Array(buckets[index]).fill(EffectsUtil_1.PATTERN_CHARACTER_TO_COLOR_MAP[character]));
    }
    createRainbowColors(message) {
        return this.createPatternColors(["1", "3", "4", "f", "v", "7", "1"], message);
    }
    createPatternBuckets(patternLength, messageLength) {
        const baseBucketSize = Math.floor(messageLength / patternLength);
        const buckets = Array(patternLength).fill(baseBucketSize);
        const remainder = messageLength % patternLength;
        const increment = Math.floor(patternLength / remainder);
        for (let i = 0; i < remainder; ++i) {
            const index = i * increment;
            ++buckets[index];
        }
        return buckets;
    }
}
exports.default = ColorEffect;
