"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const canvas_1 = require("canvas");
const path_1 = require("path");
const path = (0, path_1.resolve)(__dirname, "../assets/runescape_uf.ttf");
(0, canvas_1.registerFont)(path, { family: "RuneScape" });
const FONT_BASE_SIZE = 16;
const FONT_NAME = "RuneScape";
class Context {
    constructor(config, width = 0, height = 0) {
        this._config = config;
        this._context = (0, canvas_1.createCanvas)(width, height).getContext("2d");
        this._context.font = `${FONT_BASE_SIZE * this._config.scale}px ${FONT_NAME}`;
    }
    get context() {
        return this._context;
    }
    initialize() {
        if (this._config.hasShadow) {
            this._context.shadowColor = this._config.shadowColor;
            this._context.shadowOffsetX = this._config.scale;
            this._context.shadowOffsetY = this._config.scale;
        }
        return this;
    }
    fill({ colorEffect, fragment, frame, index, x, y }) {
        this._context.fillStyle = colorEffect.calculate({ frame, index });
        this._context.fillText(fragment, x, y);
        return this;
    }
    measureText(text) {
        const { actualBoundingBoxAscent: ascent, actualBoundingBoxDescent: descent, width, } = this._context.measureText(text);
        return {
            ascent,
            height: ascent + descent + (this._config.hasShadow ? this._config.scale : 0),
            width,
        };
    }
}
exports.default = Context;
