"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const gifenc_1 = require("gifenc");
const MAX_COLORS = 256;
class Encoder {
    constructor(config) {
        this._config = config;
    }
    encode(contexts) {
        const { height, width } = contexts[0].canvas;
        return {
            data: this.encodeGif(contexts, width, height),
            extension: "gif",
            framesCount: contexts.length,
            height,
            width,
        };
    }
    encodeGif(contexts, width, height) {
        const gif = (0, gifenc_1.GIFEncoder)();
        contexts.forEach((context) => {
            const data = context.getImageData(0, 0, width, height).data;
            const palette = (0, gifenc_1.quantize)(data, MAX_COLORS, {
                format: this._config.format,
            });
            const index = (0, gifenc_1.applyPalette)(data, palette, this._config.format);
            gif.writeFrame(index, width, height, {
                delay: this._config.delayPerFrame,
                palette,
                transparent: true,
            });
        });
        gif.finish();
        return gif.bytesView();
    }
}
exports.default = Encoder;
