"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ColorEffect_1 = __importDefault(require("./classes/ColorEffect"));
const Encoder_1 = __importDefault(require("./classes/Encoder"));
const Logger_1 = __importDefault(require("./classes/Logger"));
const MotionEffect_1 = __importDefault(require("./classes/MotionEffect"));
const Parser_1 = __importDefault(require("./classes/Parser"));
const Renderer_1 = __importDefault(require("./classes/Renderer"));
const ConfigUtil_1 = require("./utils/ConfigUtil");
function getRuneScapeText(string, options, wordWrapOptions) {
    const config = (0, ConfigUtil_1.getConfig)(options);
    const wordWrapConfig = (0, ConfigUtil_1.getWordWrapConfig)(wordWrapOptions);
    const logger = new Logger_1.default(config.debug);
    const { colorEffect, encoder, motionEffect, parser, renderer } = logger.time("Initializing", () => {
        const parser = new Parser_1.default(config, wordWrapConfig);
        const colorEffect = new ColorEffect_1.default(config);
        const motionEffect = new MotionEffect_1.default(config);
        const renderer = new Renderer_1.default(config);
        const encoder = new Encoder_1.default(config);
        return { colorEffect, encoder, motionEffect, parser, renderer };
    });
    const { color, message, motion, pattern } = logger.time("Parsing", () => {
        return parser.parse(string);
    });
    logger.time("Setting", () => {
        colorEffect.setColor(color, pattern, message);
        motionEffect.setMotion(motion);
        renderer.setEffects(colorEffect, motionEffect);
    });
    const contexts = logger.time("Rendering", () => {
        return renderer.render(message);
    });
    const response = logger.time("Encoding", () => {
        return encoder.encode(contexts);
    });
    return Object.assign(Object.assign({}, response), { color, message, motion, pattern });
}
exports.default = getRuneScapeText;
